#ifndef BUFFERPOOL_H
#define BUFFERPOOL_H

#include "MsgSS.h" //For MAX_MSGSS_PAYLOAD_SIZE

#define BUFFER_SIZE MAX_MSGSS_PAYLOAD_SIZE
#define POOL_SIZE 5

#define atomic_ph if(1)

struct buffer_item {
  int8_t data[BUFFER_SIZE];
  struct buffer_item* next;
};

typedef struct buffer_item bp_item;

bp_item bp_pool[POOL_SIZE];   // pool of buffers
bp_item* bp_free_stack_head;  // stack of unallocated buffers

/*
  Protos
*/
inline void bp_init();
inline void bp_stat();
inline bp_item* bp_alloc();
inline void bp_free(bp_item* b);
inline void bp_free_all(bp_item* b);


/*
  Implementation
*/

/*
  Inizializza il buffer pool

*/
inline void bp_init() {
  uint8_t i,j;
  for (i=0; i<POOL_SIZE; i++) {
    for (j=0; j<BUFFER_SIZE; j++) {
      bp_pool[i].data[j]=0;
    }
    bp_pool[i].next = &(bp_pool[i+1]);
  }
  bp_pool[POOL_SIZE-1].next = NULL;
  bp_free_stack_head = &(bp_pool[0]);
  dbg(DBG_USR2, "bp_init: buffer pool begins at %d, %d bytes used on memory per buffer\n", &(bp_pool[0]), ((int) &(bp_pool[1])) - ((int)&(bp_pool[0])));
}

/*
  Mostra lo stato attuale del buffer pool
*/
inline void bp_stat() {
  bp_item* t = bp_free_stack_head;
  uint8_t i = 0;
  
  while (t != NULL) {
    t = t->next;
    i++;
  }
  dbg(DBG_USR2, "bp_stat: %d of %d allocated buffers (%d free buffers left)\n", POOL_SIZE - i, POOL_SIZE, i);
}

/*
  Alloca un buffer item dal buffer pool e lo inizializza.

  @return NULL in caso di errore, il puntatore al nuovo buffer altrimenti

*/
inline bp_item* bp_alloc() {
  bp_item* b;
  uint8_t j;
  
  atomic_ph {    //FIXME Forse non serve
		if (bp_free_stack_head == NULL) {
      dbg(DBG_USR2, "bp_alloc: bp_free_stack_head is null, no free buffers left\n");
      return NULL;
		}

		dbg(DBG_USR2, "bp_alloc: allocating a new buffer: bp_free_stack_head points to %d\n", bp_free_stack_head);

		b = bp_free_stack_head;
		bp_free_stack_head = bp_free_stack_head->next;
		b->next = NULL;

		dbg(DBG_USR2, "bp_alloc: b points to %d (next is %d) and bp_free_stack_head points to %d\n", b, b->next, bp_free_stack_head);
  }

  for (j=0; j<BUFFER_SIZE; j++) 
    b->data[j] = 0;
  
  dbg(DBG_USR2, "bp_alloc: completed (returns %d)\n", b);
  return b;
}

/*
  Rilascia un buffer

  @param b Puntatore al buffer da rilasciare

*/
inline void bp_free(bp_item* b) {
  dbg(DBG_USR2, "bp_free: freeing buffer at %d...\n", b);

	if (b == NULL)
		return;
		
  atomic_ph { //FIXME Forse non serve
	  dbg(DBG_USR2, "bp_free: b->next points to %d, bp_free_stack_head points to %d...\n", b->next, bp_free_stack_head);
    b->next = bp_free_stack_head;
		bp_free_stack_head = b;
		dbg(DBG_USR2, "bp_free: now b->next points to %d and bp_free_stack_head points to %d\n[NEWCODE] bp_free: done\n", b->next, bp_free_stack_head);
  }
  
}

/*
  Rilascia il buffer richiesto e tutti i suoi successori

  @param b Puntatore al buffer da rilasciare

*/
inline void bp_free_all(bp_item* b) {
  bp_item* t;
  uint8_t i = 0;
  
  dbg(DBG_USR2, "bp_free_all: freeing all buffers starting from %d\n", b);

  atomic_ph { //FIXME Forse non serve
    while (b != NULL) {
      t = b->next;
      b->next = bp_free_stack_head;
		  bp_free_stack_head = b;
      b = t;
      i++;
    }
  }
  dbg(DBG_USR2, "bp_flush: %d buffers freed\n", i);
}

#endif
