/*
 * Management of Data in Wireless Sensor Networks (MaD-WiSe)
 *
 * Copyright 2004-2006 Consiglio Nazionale delle Ricerche
 * p.le Aldo Moro, 7 00185 Roma (IT)
 * Authors are listed in the file AUTHORS.TXT
 * This file is subject to the MIT license, to be found in LICENSE.TXT
 */

#ifndef MSGNET_H    // if not used, compiler reports error over every type
#define MSGNET_H    // declared in MsgNet.h when Network layer is used
                    // with Stream System layer because file MsgNet.h is
                    // included in TableEntry.h of SS layer


#define PAYLOAD_DATA_LENGTH 56

#include "TosTime.h"

typedef uint16_t channel_t;

enum {
  K_ADJUST_TIMER = 100 // minimum time needed to calculate and post tasks...
};

enum {
  MAX_MESSAGE_NEIGHBORS = 4 // max number of neighbors in hello_packet payload
};

enum {
  ROUTINGTABLESIZE = 4 // #entries collected in routing table of the node
};

enum {  // network layer packet types
  ASYSEND    = 0,
  CONNECT    = 1,
  SEND       = 2,
  DISCONNECT = 3,
  SETCLOCK   = 4,
  HELLO      = 5,
  ND_START   = 6,
  BUILD_TREE = 7,
  EE_START   = 8,
  EE_STOP    = 9,
  NEIGHBOR_REPORT  = 10,
  ROUTINGTABLE     = 11,
  ASYSEND_WITH_ACK = 12
};

// network layer packet header
struct header {
  uint8_t type: 5;
  uint8_t foo: 1;//empty bit
  uint8_t fuu: 1;//empty bit
  uint8_t ack: 1;//acknowledgement
  uint8_t dst;
  uint8_t src;
  uint8_t seqno;
};
typedef struct header header_t;

// entry of routingtable network layer packet
struct rt_couple {
  uint8_t id;
  uint8_t nhop;
};
typedef struct rt_couple rt_couple_t;


struct asy_payload {
  uint8_t    payload[PAYLOAD_DATA_LENGTH];
};
typedef struct asy_payload asy_payload_t;

struct setclock_payload {
  tos_time_t time;
};
typedef struct setclock_payload setclock_payload_t;

struct rtable_payload {
  rt_couple_t rt[ROUTINGTABLESIZE];
};
typedef struct rtable_payload rtable_payload_t;

struct hello_payload {
  uint8_t    neighbors[MAX_MESSAGE_NEIGHBORS];
};
typedef struct hello_payload hello_payload_t;

struct nbreport_payload {
  uint8_t    neighbors[MAX_MESSAGE_NEIGHBORS];
};
typedef struct nbreport_payload nbreport_payload_t;

struct connect_payload {
  channel_t  channel_id;
  uint8_t    data_size;
  uint32_t   stream_rate;
  uint8_t    payload[PAYLOAD_DATA_LENGTH-6];
};
typedef struct connect_payload connect_payload_t;

struct simple_payload {
  channel_t  channel_id;
  uint8_t    payload[PAYLOAD_DATA_LENGTH-1];
};
typedef struct simple_payload send_payload_t;
typedef struct simple_payload disconnect_payload_t;

// payload (field: data) of TOS_Msg packet (MAC level)
// network layer packet
struct MsgNet {
	header_t   header;
  union {
    hello_payload_t      hello_payload;
    nbreport_payload_t   nbreport_payload;
    rtable_payload_t     rtable_payload;
    setclock_payload_t   setclock_payload;
    asy_payload_t        asy_payload;
    connect_payload_t    connect_payload;
    disconnect_payload_t disconnect_payload;
    send_payload_t       send_payload;
  } payload;
} ;
typedef struct MsgNet  MsgNet;
typedef struct MsgNet* MsgNetPtr;

#endif
