/*
 * Authors:             Marco Dinarelli & Valentina Masi
 * Date last modified:  7/10/04
 *
 */

#ifndef MSGSS_H
#define MSGSS_H

#define MAX_MSGSS_PAYLOAD_SIZE 20 

#include "TosTime.h"   // contains declaration of tos_time_t

enum {                 // msg_type
	OPEN_SS = 0,     
	WRITE_SS = 1,
	CLOSE_SS = 2
};

typedef struct ss_header {
	uint8_t msg_type;
	uint16_t src;             // address of remote stream source          
	uint8_t symbolic_name;
} ss_header;

typedef struct ss_open_payload {
  uint32_t stream_rate;  
	uint8_t reliability;      // 0 = unreliable stream; 1 otherwise
} ss_open_payload;

typedef struct ss_write_payload {
  uint8_t data_size;
	int8_t data[MAX_MSGSS_PAYLOAD_SIZE];
} ss_write_payload;

// this struct is SS layer packet and Network layer payload
typedef struct MsgSS {
  ss_header header;
  union {
    ss_open_payload open;
    ss_write_payload write;
  } payload;                // Null in caso di CLOSE_SS
} MsgSS;

typedef MsgSS* MsgSSPtr;

/*

	array data structure
	
	byte 0-1:    source ID
	byte 2-3:    timestamp
	byte 4-5:    epoch
	byte 6..19:  data

*/

#endif
