/*
 * Management of Data in Wireless Sensor Networks (MaD-WiSe)
 *
 * Copyright 2004-2006 Consiglio Nazionale delle Ricerche
 * p.le Aldo Moro, 7 00185 Roma (IT)
 * Authors are listed in the file AUTHORS.TXT
 * This file is subject to the MIT license, to be found in LICENSE.TXT
 */

// AM message type for GUI commands
enum {
	AM_OPERATORMSG = 8,
	AM_OPERATORACKMSG = 12,AM_SYNCMSG = 13
};

// Enumeration for comparison operator
enum comparisonTypes {
	Equal = 1,
	NotEqual = 2,
	Greater = 3,
	GreaterEqual = 4,
	Less = 5,
	LessEqual = 6
};

// Payload layout for select(field op constant) messages
struct selectcMsgBody {
	uint8_t fieldIx;           // Index of first operand field
	uint16_t constant;         // Costant value for second operand
	uint8_t predicate;         // Select predicate as defined by
							 // enum comparisonTypes
	uint8_t inputStream;       // Input stream
	uint8_t outputStream;      // Output stream
	uint8_t fieldNum;          // Number of fields in input stream
};
typedef struct selectcMsgBody selectcMsgBody;

// Payload layout for select(field op field) messages
struct selectfMsgBody {
	uint8_t field1Ix;          // Index of first operand field
	uint8_t field2Ix;          // Index of second operand field
	uint8_t predicate;         // Select predicate as defined by enum comparison
	uint8_t inputStream;       // Input stream
	uint8_t outputStream;      // Output stream
	uint8_t fieldNum;          // Number of fields in input stream
};
typedef struct selectfMsgBody selectfMsgBody;

// Payload layout for bridge messages
struct bridgeMsgBody {
	uint8_t inputStream;       // Input stream
	uint8_t outputStream;      // Output stream
	uint8_t fieldNum;          // Number of fields in input stream
};
typedef struct bridgeMsgBody bridgeMsgBody;

// Payload layout for project messages
struct projectMsgBody {
	uint16_t projectFields;    // Bit field indicating which attributes are
							 // being projected, where a 1 (0) in position i
							 // indicates that the i-th field is (is not)
							 // projected
	uint8_t inputStream;       // Input stream
	uint8_t outputStream;      // Output stream
	uint8_t fieldNum;          // Number of fields in input stream
};
typedef struct projectMsgBody projectMsgBody;

// Payload layout for merge join message
struct mergeJoinMsgBody {
	uint8_t inputStream1;      // First input stream
	uint8_t inputStream2;      // Second input stream
	uint8_t outputStream;      // Output stream
	uint8_t fieldNum1;         // Number of fields for input stream 1
	uint8_t fieldNum2;         // Number of fields for input stream 2
};
typedef struct mergeJoinMsgBody mergeJoinMsgBody;

/***/
// Payload layout for sync join message (AGGIUNTO)
struct syncJoinMsgBody {
	uint8_t innerStream;       // Inner stream
	uint8_t outerStream;       // Outer stream
	uint8_t outputStream;      // Output stream
	uint8_t fieldNum1;         // Number of fields for input stream 1
	uint8_t fieldNum2;         // Number of fields for input stream 2
};
typedef struct syncJoinMsgBody syncJoinMsgBody;
/***/

// Payload layout for sync message
struct syncMsgBody {
	uint16_t timestamp;
};
typedef struct syncMsgBody syncMsgBody;

// Time aggregate types
enum timeAggregateValues {
	TUnused  = 0,              // Invalid
	TMax 	 = 1,              // Max
	TMin 	 = 2,              // Min
	TAverage = 3,              // Average
	TCount 	 = 4               // Count
};

// Time aggregate description field
struct timeAggregateField {
	uint8_t type:3;            // Aggregate type (as defined in enum timeAggregateValues)
	uint8_t fieldIx:5;         // Index of field the aggregate refers to
};
typedef struct timeAggregateField timeAggregateField;

// Maximum number of time aggregates
enum {
	MaxTAggregates = 4
};

// Payload layout for time aggregate messages
struct timeAggregateMsgBody {
	uint8_t inputStream;            // Input stream
	uint8_t outputStream;           // Output stream
	uint8_t fieldNum;               // Number of fields in input stream
	uint16_t epoch;                 // Number of timestamps for each aggregate calculation
	timeAggregateField aggregates[MaxTAggregates];  // Aggregates
};
typedef struct timeAggregateMsgBody timeAggregateMsgBody;

// Space aggregate types
enum spaceAggregateValues {
	SMax = 1,                  // Max
	SMin = 2,                  // Min
	SPartialAverage = 3,       // Partial average
	SFinalAverage 	= 4,       // Final average
	SCount = 5                 // Count
};

// Space aggregate description field
struct spaceAggregateField {
	uint8_t mult:1;            // Multiplicity of input stream (1 = yes, 0 = no)
	uint8_t fieldIx:7;         // Index of field the aggregate refers to
};
typedef struct spaceAggregateField spaceAggregateField;

// Payload layout for space aggregate messages
struct spaceAggregateMsgBody {
	uint8_t inputStream1;           // First input stream
	uint8_t inputStream2;           // Second input stream
	uint8_t outputStream;           // Output stream
	uint8_t fieldNum1;              // Number of fields for input stream 1
	uint8_t fieldNum2;              // Number of fields for input stream 2
	spaceAggregateField aggregate1; // Aggregate description for input stream 1
	spaceAggregateField aggregate2; // Aggregate description for input stream 2
};
typedef struct spaceAggregateMsgBody spaceAggregateMsgBody;

// Payload layout for serial messages
struct serialMsgBody {
	uint8_t inputStream;       // Input stream
	uint8_t fieldNum;          // Number of fields in input stream
};
typedef struct serialMsgBody serialMsgBody;

// Payload layout for union Messages
struct unionMsgBody {
	uint8_t inputStream1;
	uint8_t fieldNum1;
	uint8_t inputStream2;
	uint8_t fieldNum2;
	uint8_t outputStream;
};
typedef struct unionMsgBody unionMsgBody;

// Payload layout for start query messages
struct startMsgBody {
};
typedef struct startMsgBody startMsgBody;


// Payload layout for stop query messages
struct stopMsgBody {
};
typedef struct stopMsgBody stopMsgBody;


// Payload layout for remote open messages
struct remoteOpenMsgBody {
	uint8_t symbolicName;      // Symbolic name for remote stream
	uint32_t streamRate;       // Stream rate
  uint32_t startTime;
	uint8_t reliability;       // Stream reliability
	uint8_t bufferSize;        // Stream buffer size
	uint8_t streamIx;          // Stream index on sensor
	uint16_t remoteEnd;        // Remote end
};
typedef struct remoteOpenMsgBody remoteOpenMsgBody;

// Sampling types
enum samplingTypes {
	Periodic = 1,
	OnDemand = 2
};

// Transducer types
enum transducerTypes {
	Light 		  = 1,
	Temperature   = 2,
	Audio 		  = 3,
	MagnetismX 	  = 4,
	MagnetismY 	  = 5,
	AccelerationX = 6,
	AccelerationY = 7
};

// Payload layout for sensor open messages
struct sensorOpenMsgBody {
	uint8_t samplingType;     // Sampling type (as defined in enum samplingTypes)
	uint32_t streamRate;      // Sampling rate
	uint8_t bufferSize;       // Buffer size
	uint8_t streamIx;         // Stream index on sensor
	uint8_t sensorType;       // Transducer (as defined in enum transducerTypes)
};
typedef struct sensorOpenMsgBody sensorOpenMsgBody;

// Payload layout for local open messages
struct localOpenMsgBody {
	uint8_t bufferSize;       // Buffer size
	uint8_t streamIx;         // Stream index on sensor
};
typedef struct localOpenMsgBody localOpenMsgBody;

// Command message body layout
union msgBody {
	selectcMsgBody    selectC;
	selectfMsgBody    selectF;
	bridgeMsgBody 	  bridge;
	projectMsgBody    project;
	mergeJoinMsgBody  mergeJoin;
	syncJoinMsgBody   syncJoin;
	unionMsgBody  	  unionOp;
	serialMsgBody     serial;
	startMsgBody      start;
	stopMsgBody       stop;
	remoteOpenMsgBody remoteOpen;
	sensorOpenMsgBody sensorOpen;
	localOpenMsgBody  localOpen;
	timeAggregateMsgBody  timeAggregate;
	spaceAggregateMsgBody spaceAggregate;
  syncMsgBody       syncBody;//aggiunto
};
typedef union msgBody msgBody;

// Message types
enum messageTypes {
	SelectC 		  = 1,
	SelectF 		  = 2,
	TimeAggregate = 3,
	SyncJoin 		  = 4,
	Count 			  = 5,
	MergeJoin 		= 6,
	Project 		  = 7,
	Bridge 			  = 8,
	RemoteOpen 		= 9,
	LocalOpen 		= 10,
	SensorOpen 		= 11,
	StartQuery 		= 12,
	StopQuery 		= 13,
	Union 			  = 14,
	Serial 			  = 15,
	Min 			    = 16,
	Max 			    = 17,
	PartialAverage= 18,
	Average 		  = 19,
	SpaceAggregate= 20,
  Data          = 21
};

// Operator message layout
struct OperatorMsg {
	uint8_t queryId;           // Query id
	uint8_t msgType;           // Message type
	msgBody body;              // Message body
};
typedef struct OperatorMsg OperatorMsg;

// Operator ack message layout
struct OperatorAckMsg {
  uint8_t queryId;           // Query id
  uint8_t msgType;           // Message type
  uint8_t streamIx;          // Stream index
};
typedef struct OperatorAckMsg AckMsg;
