#ifndef SSTYPES_H
#define SSTYPES_H

#include "BufferPool.h"

// stream descriptor type definition: table identifier + table entry index
typedef struct stream_type {
	uint8_t table_id;                     
	int8_t no;       // max 4 entries for each table  
} stream_type;

typedef int8_t** buffer_type;  // each element is a pointer to a buffer which
                               // can contain at most MAX_MSGSS_PAYLOAD_SIZE
                               // byte; buffer size is fixed when open_r is
                               // being performed

typedef struct read_request {
	uint8_t nbytes;
	//stream_type stream_desc; // unnecessary because every entry in table has
                             // its own array read_requested
} read_request;

typedef struct stream_buffer {
  bp_item* bp_rpos;                 // pointer to the first buffer item to read in the Buffer Pool
  bp_item* bp_wpos;                 // pointer to the last buffer item written in the Buffer Pool
  uint8_t record_size;              // size (in bytes) of a record travelling on this stream
  uint8_t pending_reads;            // number of pending reads of record_size size from this stream
  uint8_t buffer_size;              // max length of buffer queue allowed
  uint8_t buffers_used;
} stream_buffer;

const uint8_t madwise_string[] = {0x6d, 0x61, 0x64, 0x77, 0x69, 0x73, 0x65, 0x0a};
typedef uint8_t qname[7];
typedef struct sstor_header { //MUST BE EXACTLY TOS_EEPROM_LINE_SIZE BYTES!!!
  uint8_t   node;
  uint8_t   record_size;
  uint16_t  rpos;
  uint16_t  wpos;
  uint16_t  stored_records;
  uint8_t   checksum;
  qname     name;
} sstor_header;

enum {
  EE_NOT_OPENED = 0,
  EE_READY = 1,
  EE_READ_SINGLE = 2,
  EE_READ_FIRST = 3,
  EE_READ_SECOND = 4,
  EE_READ_HEADER = 5,
  EE_WRITE_SINGLE = 6,
  EE_WRITE_FIRST = 7,
  EE_WRITE_SECOND = 8,
  EE_WRITE_EMPTY_HEADER = 9,
  EE_WRITE_HEADER = 10
};


#endif
