/*
 * Management of Data in Wireless Sensor Networks (MaD-WiSe)
 *
 * Copyright 2004-2006 Consiglio Nazionale delle Ricerche
 * p.le Aldo Moro, 7 00185 Roma (IT)
 * Authors are listed in the file AUTHORS.TXT
 * This file is subject to the MIT license, to be found in LICENSE.TXT
 */


#include "TableEntry.h"  //contains stream_type declaration
#include "OperatorMsg.h"
#define MAX_FIELDS 10
                      // it would be at least equal to the number of
                      // variables sampled from physical environment + 1
                      // (timestamp), and at most equal to
                      // Stream System packet size / 2

#define MAX_OPERATORS 12
#define MAX_OPENS 10
#define MAX_ELEMENTS_QUEUE 10
#define MAX_ACK_QUEUE 5
#define MAX_QUERYS 5

typedef struct open_entry{
	stream_type stream_desc;
	uint8_t symbolic_name;     // only for remote stream
	uint8_t sampling_type;     // only for sensor stream
	uint8_t validity;
} open_entry;

typedef struct query_open_table
{
	uint8_t query_id;
	open_entry stream_table[MAX_OPENS];
} query_open_table;

/*
validity = 0                         -->  empty entry
stream_desc.no = -1                  -->  stream not opened yet
sensor_tid != 0                      -->  sensor stream
symbolic_name > 0                    -->  remote stream
sensor_tid = 0 && symbolic_name = 0  -->  local stream
*/

typedef struct record_type {
  uint8_t queryId;
  uint8_t type;
  uint8_t numfields; // actual size of data_fields array (from 1 to MAX_FIELDS)
  union {
    int8_t  buffer[MAX_FIELDS*2];
    int16_t wbuff[MAX_FIELDS];
  } array_t;
} record_type;


/*typedef enum predicate_type{
    EQUAL = 1,
    NOT_EQUAL = 2,
    GREATER = 3,
    GREATER_OR_EQUAL = 4,
    LESS = 5,
    LESS_OR_EQUAL = 6
} predicate_type;


typedef enum operator_type{
    SELECT_C = 1,
    SELECT_F = 2,
    AGGREGATE_T = 3,
    SERIAL_SEND = 4,
    MERGE_JOIN = 5,
    SYNC_JOIN = 6,
    BRIDGE = 7,
    PROJECTION = 8,
    AGGREGATE_S = 9
} operator_type;


typedef enum aggregate_type{
    MAX = 1,
    MIN = 2,
    AVERAGE = 3,
    SUM = 4,
    AVERAGE_P =5
} aggregate_type;

*/

typedef struct select_c_row
{
  uint8_t field;
  uint8_t constant;
  uint8_t predicate; // Select predicate as defined by
                       // enum comparisonTypes
} select_c_row;

typedef struct select_f_row
{
  uint8_t field1;
  uint8_t field2;
  uint8_t predicate; // Select predicate as defined by
                       // enum comparisonTypes
} select_f_row;

typedef struct aggregate_t_struct
{
	uint8_t name:3; // Aggregate type (as defined in enum timeAggregateValues)
	uint8_t field:5;
	int16_t value;
	uint8_t molt;
} aggregate_t_struct;

typedef struct agg_t_row
{
  aggregate_t_struct aggregate_op[MaxTAggregates];
  uint16_t epoch;
  uint16_t last_ts; //last entry timestamp
  int8_t flag;
} agg_t_row;

typedef struct agg_s_row
{
	record_type record;
	uint8_t opName;
	uint8_t flagInput1;
	uint8_t flagInput2;

	spaceAggregateField aggregate1;
	spaceAggregateField aggregate2;

} agg_s_row;

typedef struct join_row
{
	record_type record;
	uint8_t flag;
} join_row;

typedef struct projection_row
{
   uint16_t projectFields;
} projection_row;


typedef struct op_struct
{
	uint8_t query_id;
	uint8_t operator;  // operator type (as defined in enum MessageTypes)
	uint8_t stream_input:4;
	uint8_t stream_input2:4;
	uint8_t stream_output:4;
	uint8_t fieldNum:4;
	uint8_t fieldNum2:4;

	union
	{
    join_row join;
    select_c_row select_c;
    select_f_row select_f;
    agg_t_row agg_t;
    projection_row projection;
    agg_s_row agg_s;
	} operator_data;

} op_struct;

typedef struct
{
	uint8_t validity;
	record_type record;
	stream_type stream_desc;
} rd_parameters;

typedef struct
{
	uint8_t num_elements; //num elements in queue
	uint8_t tail;
	uint8_t head;
	rd_parameters elements[MAX_ELEMENTS_QUEUE];
} rd_queue;

typedef struct
{
	uint8_t num_elements; //num elements in queue
	uint8_t tail;
	uint8_t head;
	OperatorMsg msg[MAX_ELEMENTS_QUEUE];
} cc_queue;
